@@ -17,6 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "defns.h"
+
+#include <locale.h>
 
 #include "capplet.h"
 #include "colourgui.h"
@@ -309,11 +312,23 @@ int main(int argc, char **argv)
 	gboolean persist = FALSE;
 	gboolean open_configlet = FALSE;
 	gboolean dbus_ok;
+    GdkScreen *scrn;
+    char *disp_name;
 
 	g_set_application_name(_("roxterm-config"));
 
 	global_options_init_appdir(argc, argv);
+
+#if ENABLE_NLS
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, global_options_appdir ?
+			g_strdup_printf("%s/locale", global_options_appdir) : LOCALEDIR);
+	textdomain(PACKAGE);
+#endif	
+
 	gtk_init(&argc, &argv);
+    scrn = gdk_screen_get_default();
+    disp_name = gdk_screen_make_display_name(scrn);
 
 	logo_filename = logo_find();
 	gtk_window_set_default_icon_from_file(logo_filename, NULL);
@@ -341,17 +356,20 @@ int main(int argc, char **argv)
 		/* Defer to another instance */
 		if (profile)
 		{
-			if (optsdbus_send_edit_profile_message(profile))
+			if (optsdbus_send_edit_profile_message(profile, disp_name))
 				profile = NULL;
 		}
 		if (colour_scheme)
 		{
-			if (optsdbus_send_edit_colour_scheme_message(colour_scheme))
+			if (optsdbus_send_edit_colour_scheme_message(colour_scheme,
+			        disp_name))
+	        {
 				colour_scheme = NULL;
+			}
 		}
 		if (open_configlet)
 		{
-			if (optsdbus_send_edit_opts_message("Configlet", ""))
+			if (optsdbus_send_edit_opts_message("Configlet", "", disp_name))
 			{
 				open_configlet = FALSE;
 			}
@@ -360,23 +378,25 @@ int main(int argc, char **argv)
 
 	if (profile)
 	{
-		profilegui_open(profile);
+		profilegui_open(profile, scrn);
 		persist = TRUE;
 	}
 	if (colour_scheme)
 	{
-		colourgui_open(colour_scheme);
+		colourgui_open(colour_scheme, scrn);
 		persist = TRUE;
 	}
 	if (open_configlet)
 	{
-		if (configlet_open())
+		if (configlet_open(scrn))
 			persist = TRUE;
 	}
 
 	if (dbus_ok && listen)
 		persist = TRUE;
 
+    g_free(disp_name);
+    
 	if (persist)
 		gtk_main();
 
