@@ -30,7 +30,7 @@ typedef struct ColourGUI ColourGUI;
 
 /* Loads a Colour and creates a working dialog box for it.
  * Creates new colour scheme if it doesn't already exist */
-ColourGUI *colourgui_open(const char *colour_scheme_name);
+ColourGUI *colourgui_open(const char *colour_scheme_name, GdkScreen *scrn);
 
 void colourgui_delete(ColourGUI *);
 
