@@ -198,7 +198,8 @@ int colour_scheme_get_palette_size(Options * opts)
 	return scheme->palette_size;
 }
 
-GdkColor *colour_scheme_get_cursor_colour(Options * opts, gboolean def)
+GdkColor *colour_scheme_get_cursor_colour(Options *opts,
+        gboolean allow_null)
 {
 	ColourScheme *scheme;
 
@@ -210,7 +211,7 @@ GdkColor *colour_scheme_get_cursor_colour(Options * opts, gboolean def)
 	{
 		scheme->cursor = g_new0(GdkColor, 1);
 		if (!colour_scheme_lookup_and_parse(opts, scheme, scheme->cursor,
-					"cursor", def ? "black" : NULL, TRUE))
+					"cursor", allow_null ? NULL : "#c0c0c0", TRUE))
 		{
 			g_free(scheme->cursor);
 			scheme->cursor = NULL;
@@ -219,7 +220,8 @@ GdkColor *colour_scheme_get_cursor_colour(Options * opts, gboolean def)
 	return scheme->cursor;
 }
 
-GdkColor *colour_scheme_get_foreground_colour(Options * opts, gboolean def)
+GdkColor *colour_scheme_get_foreground_colour(Options * opts,
+            gboolean allow_null)
 {
 	ColourScheme *scheme;
 
@@ -231,7 +233,7 @@ GdkColor *colour_scheme_get_foreground_colour(Options * opts, gboolean def)
 	{
 		scheme->foreground = g_new0(GdkColor, 1);
 		if (!colour_scheme_lookup_and_parse(opts, scheme, scheme->foreground,
-				"foreground", def ? "white" : NULL, FALSE))
+				"foreground", allow_null ? NULL : "#c0c0c0", FALSE))
 		{
 			g_free(scheme->foreground);
 			scheme->foreground = NULL;
@@ -240,7 +242,8 @@ GdkColor *colour_scheme_get_foreground_colour(Options * opts, gboolean def)
 	return scheme->foreground;
 }
 
-GdkColor *colour_scheme_get_background_colour(Options * opts, gboolean def)
+GdkColor *colour_scheme_get_background_colour(Options * opts,
+        gboolean allow_null)
 {
 	ColourScheme *scheme;
 
@@ -252,7 +255,7 @@ GdkColor *colour_scheme_get_background_colour(Options * opts, gboolean def)
 	{
 		scheme->background = g_new0(GdkColor, 1);
 		if (!colour_scheme_lookup_and_parse(opts, scheme, scheme->background,
-				"background", def ? "black" : NULL, FALSE))
+				"background", allow_null ? NULL : "#000", FALSE))
 		{
 			g_free(scheme->background);
 			scheme->background = NULL;
