@@ -34,13 +34,15 @@ int colour_scheme_get_palette_size(Options * opts);
  * smaller */
 GdkColor *colour_scheme_get_palette(Options * opts);
 
-/* If def is TRUE, a default colour is used if the option isn't set, otherwise
- * it returns NULL */
-GdkColor *colour_scheme_get_cursor_colour(Options * opts, gboolean def);
+/* If allow_null is FALSE a default colour is used */
+GdkColor *colour_scheme_get_cursor_colour(Options * opts,
+        gboolean allow_null);
 
-GdkColor *colour_scheme_get_foreground_colour(Options * opts, gboolean def);
+GdkColor *colour_scheme_get_foreground_colour(Options * opts,
+        gboolean allow_null);
 
-GdkColor *colour_scheme_get_background_colour(Options * opts, gboolean def);
+GdkColor *colour_scheme_get_background_colour(Options * opts,
+        gboolean allow_null);
 
 void colour_scheme_set_palette_size(Options * opts, int size);
 
