@@ -105,7 +105,7 @@ static const char *full_name_from_family(const char *family)
 	}
 	else if (!strcmp(family, "Shortcuts"))
 	{
-		return _("Keyboard Accelerator Scheme");
+		return _("Keyboard Shortcuts Scheme");
 	}
 	else if (!strcmp(family, "encodings"))
 	{
