@@ -33,6 +33,7 @@
 #define _(s) ((const char *) gettext(s))
 #else
 #define _(s) s
+#define gettext(s) s
 #endif
 #define N_(s) s
 
@@ -52,17 +53,19 @@
 
 /*
 #define UNREF_LOG(a) g_debug("Calling ` %s ' from %s", #a, __func__); \
-		a ; \
-		g_debug("unref complete");
+        a ; \
+        g_debug("unref complete");
 */
 #define UNREF_LOG(a) a
 
 #define ROXTERM_ARG_ERROR g_quark_from_static_string("roxterm-arg-error-quark")
 
 enum {
-	ROXTermArgError
+    ROXTermArgError
 };
 
+#define ROXTERM_LEAF_DIR "roxterm.sourceforge.net"
+
 #endif /* DEFNS_H */
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
