@@ -30,6 +30,9 @@ void dlg_message(GtkWindow *parent, const char *, ...);
 void dlg_warning(GtkWindow *parent, const char *, ...);
 void dlg_critical(GtkWindow *parent, const char *, ...);
 
+GtkWidget *dlg_ok_cancel(GtkWindow *parent, const char *title,
+        const char *, ...);
+
 #endif /* CONFIGLET_H */
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
