@@ -45,9 +45,17 @@ extern char *global_options_directory;
 /* Whether to open next window fullscreen */
 extern gboolean global_options_fullscreen;
 
+/* Whether to open next window maximised */
+extern gboolean global_options_maximise;
+
 /* Whether to try to open new terminal in an existing window */
 extern gboolean global_options_tab;
 
+/* For session management */
+extern gboolean global_options_disable_sm;
+extern char *global_options_restart_session_id;
+extern char *global_options_clone_session_id;
+
 /* Call after argv has been processed by gtk_init; may be called more than once
  * but repeat invocations have no effect on appdir/bindir and argv/argc are
  * altered. Bear in mind that --help/--usage args will cause exit.
