@@ -25,8 +25,12 @@
 
 #include "dlg.h"
 #include "globalopts.h"
+#include "multitab.h"
 #include "roxterm.h"
 #include "rtdbus.h"
+#if ENABLE_SM
+#include "session.h"
+#endif
 
 /* List of all terminal windows managed by this app */
 static GList *main_all_windows = NULL;
@@ -131,14 +135,11 @@ static DBusHandlerResult new_term_listener(DBusConnection *connection,
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;    
     }
     argv = rtdbus_get_message_args_as_strings(message);
-    g_print("Args:\n");
     for (argc = 0; argv && argv[argc]; ++argc)
     {
-        g_print("  %s\n", argv[argc]);
         if (g_str_has_prefix(argv[argc], "--display="))
             display = argv[argc] + 10;
     }
-    g_print("\n");
     if (argc)
     {
         global_options_preparse_argv_for_execute(&argc, argv, TRUE);
@@ -160,14 +161,24 @@ gboolean listen_for_new_term(void)
             new_term_listener, global_options_lookup_int("replace") > 0);
 }
 
+static const char *abs_bin(const char *filename)
+{
+    if (g_path_is_absolute(filename))
+    {
+        return filename;
+    }
+    return g_build_filename(global_options_bindir, "roxterm", NULL);
+}
+
 int main(int argc, char **argv)
 {
-    gboolean dbus_ok;
     gboolean preparse_ok;
-    DBusMessage *message;
+    DBusMessage *message = NULL;
     char *display = NULL;
     const char *dpy_name;
     int n;
+    gboolean launched = FALSE;
+    gboolean dbus_ok;
 
     g_set_application_name(PACKAGE);
     preparse_ok = global_options_preparse_argv_for_execute(&argc, argv, FALSE);
@@ -203,8 +214,20 @@ int main(int argc, char **argv)
         display = g_strdup_printf("--display=%s", dpy_name);
     
     /* Have to create message with args from argv before parsing them */
-    dbus_ok = rtdbus_init();
-    message = create_dbus_message(argc, argv, display);
+    dbus_ok = rtdbus_ok = rtdbus_init();
+    if (dbus_ok)
+    {
+#if ENABLE_SM
+        if (global_options_disable_sm ||
+                (!global_options_restart_session_id &&
+                !global_options_clone_session_id))
+        {
+            message = create_dbus_message(argc, argv, display);
+        }
+#else
+        message = create_dbus_message(argc, argv, display);
+#endif
+    }
     g_free(display);
     
     global_options_init(&argc, &argv, TRUE);
@@ -214,8 +237,16 @@ int main(int argc, char **argv)
         dbus_ok = listen_for_new_term();
         /* Only TRUE if another roxterm is providing the service */
     }
+    
+    dbus_ok = global_options_lookup_int("separate") <= 0 && dbus_ok
+#if ENABLE_SM
+            && (global_options_disable_sm ||
+                (!global_options_restart_session_id
+                && !global_options_clone_session_id))
+#endif
+            ;
 
-    if (global_options_lookup_int("separate") <= 0 && dbus_ok)
+    if (dbus_ok)
     {
         switch (run_via_dbus(message))
         {
@@ -235,11 +266,46 @@ int main(int argc, char **argv)
 
     gtk_quit_add(0, main_quit_handler, main_all_windows);
 
-    roxterm_init(dpy_name);
+    roxterm_init();
+#if ENABLE_SM
+    if (!global_options_disable_sm)
+    {
+        if (global_options_restart_session_id)
+            launched = session_load(global_options_restart_session_id);
+        if (!launched && global_options_clone_session_id)
+            launched = session_load(global_options_clone_session_id);
+    }
+#endif
+    multi_win_set_role_prefix("roxterm");
+    if (!launched)
+    {
+        roxterm_launch(dpy_name, NULL);
+    }
+
+#if ENABLE_SM
+    if (!global_options_disable_sm)
+    {
+        if (global_options_appdir)
+        {
+            session_arg0 = g_build_filename(global_options_appdir,
+                    "AppRun", NULL);
+            if (!g_file_test(session_arg0, G_FILE_TEST_IS_EXECUTABLE))
+            {
+                g_free((char *) session_arg0);
+                session_arg0 = abs_bin(argv[0]);
+            }
+        }
+        else
+        {
+            session_arg0 = abs_bin(argv[0]);
+        }
+        session_init(global_options_restart_session_id);
+    }
+#endif
 
     gtk_main();
 
     return 0;
 }
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
