@@ -29,10 +29,13 @@
 
 #define HAVE_COMPOSITE GTK_CHECK_VERSION(2, 10, 0)
 
+#define HORIZ_TAB_WIDTH 120
+
 struct MultiTab {
     MultiWin *parent;
     GtkWidget *widget;            /* Top-level widget in notebook */
-    char *title;
+    char *window_title;
+    char *icon_title;
     char *name;
     GtkWidget *popup_menu_item, *menu_bar_item;
     GtkWidget *active_widget;
@@ -49,6 +52,10 @@ struct MultiTab {
     GtkWidget *close_button;
     GtkWidget *label_box;
     void (*label_packer)(GtkBox *, GtkWidget *, gboolean, gboolean, guint);
+    const char *status_stock;
+    GtkWidget *image;
+    GtkWidget *rename_dialog;
+    gboolean postponed_free;
 };
 
 struct MultiWin {
@@ -95,6 +102,7 @@ struct MultiWin {
     char *title_template;
     char *default_title_template;
     char *child_title;
+    char *child_icon;
     gboolean title_override;
 #if HAVE_COMPOSITE
     gboolean composite;
@@ -122,6 +130,8 @@ static double multi_win_zoom_factors[] = {
 #define MULTI_WIN_NORMAL_ZOOM_INDEX 7
 #define MULTI_WIN_N_ZOOM_FACTORS 15
 
+static char *multi_win_role_prefix = NULL;
+static int multi_win_role_index = 0;
 
 static MultiTabFiller multi_tab_filler;
 static MultiTabDestructor multi_tab_destructor;
@@ -130,7 +140,7 @@ static MultiWinGeometryFunc multi_win_geometry_func;
 static MultiWinSizeFunc multi_win_size_func;
 static MultiTabToNewWindowHandler multi_tab_to_new_window_handler;
 static MultiWinZoomHandler multi_win_zoom_handler;
-static MultiWinGetDisableMenuShortcuts multi_win_get_disable_menu_shortcuts;
+MultiWinGetDisableMenuShortcuts multi_win_get_disable_menu_shortcuts;
 static MultiWinInitialTabs multi_win_initial_tabs;
 static MultiWinDeleteHandler multi_win_delete_handler;
 static MultiTabGetShowCloseButton multi_tab_get_show_close_button;
@@ -150,6 +160,8 @@ static void multi_win_destructor(MultiWin *win, gboolean destroy_widgets);
 
 static void multi_win_set_title(MultiWin *win, const char *title);
 
+static void multi_win_set_icon_title(MultiWin *win, const char *title);
+
 inline static MultiTab *multi_tab_from_widget(GtkWidget *widget)
 {
     return g_object_get_data(G_OBJECT(widget), "roxterm_tab");
@@ -192,16 +204,20 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     multi_tab_get_show_close_button = get_show_close_button;
 }
 
-MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
+MultiTab *multi_tab_new0(MultiWin * parent, gpointer user_data_template,
+        gboolean make_current)
 {
     MultiTab *tab = g_new0(MultiTab, 1);
 
+    tab->status_stock = NULL;
     tab->widget = multi_tab_filler(parent, tab, user_data_template,
-        &tab->user_data, &tab->title, &tab->active_widget, &tab->adjustment);
+        &tab->user_data, &tab->window_title, &tab->active_widget,
+        &tab->adjustment);
     g_object_set_data(G_OBJECT(tab->widget), "roxterm_tab", tab);
 
     multi_win_add_tab(parent, tab, -1, FALSE);
-    multi_win_select_tab(parent, tab);
+    if (make_current)
+        multi_win_select_tab(parent, tab);
 
     return tab;
 }
@@ -209,11 +225,22 @@ MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
 static void multi_tab_delete_without_notifying_parent(MultiTab * tab,
         gboolean destroy_widgets)
 {
+    if (tab->rename_dialog)
+    {
+        gtk_widget_destroy(tab->rename_dialog);
+        tab->postponed_free = TRUE;
+    }
+    else
+    {
+        tab->postponed_free = FALSE;
+    }
     (*multi_tab_destructor) (tab->user_data);
     tab->user_data = NULL;
 
-    g_free(tab->title);
+    g_free(tab->window_title);
     g_free(tab->name);
+    tab->window_title = NULL;
+    tab->name = NULL;
     if (destroy_widgets && tab->widget)
     {
         gtk_widget_destroy(tab->widget);
@@ -238,7 +265,8 @@ static void multi_tab_delete_without_notifying_parent(MultiTab * tab,
         tab->tab_drag = NULL;
     }
 #endif
-    g_free(tab);
+    if (!tab->postponed_free)
+        g_free(tab);
 }
 
 void multi_tab_delete(MultiTab * tab)
@@ -298,30 +326,57 @@ static void multi_tab_set_name_or_title(MultiTab *tab, const char *title)
     }
 }
 
-void multi_tab_set_title(MultiTab * tab, const char *title)
+void multi_tab_set_window_title(MultiTab * tab, const char *title)
 {
     MultiWin *win = tab->parent;
 
-    g_free(tab->title);
-    tab->title = title ? g_strdup(title) : NULL;
+    g_free(tab->window_title);
+    tab->window_title = title ? g_strdup(title) : NULL;
     if (win->current_tab == tab)
     {
-        multi_win_set_title(win, STR_EMPTY(tab->title));
+        multi_win_set_title(win, STR_EMPTY(tab->window_title));
     }
     if (!tab->name)
         multi_tab_set_name_or_title(tab, title);
 }
 
+void multi_tab_set_icon_title(MultiTab * tab, const char *title)
+{
+    MultiWin *win = tab->parent;
+
+    g_free(tab->icon_title);
+    tab->icon_title = title ? g_strdup(title) : NULL;
+    if (win->current_tab == tab)
+    {
+        multi_win_set_icon_title(win, tab->icon_title);
+    }
+}
+
 void multi_tab_set_name(MultiTab * tab, const char *name)
 {
     g_free(tab->name);
     tab->name = name ? g_strdup(name) : NULL;
-    multi_tab_set_name_or_title(tab, name ? name : tab->title);
+    multi_tab_set_name_or_title(tab, name ? name : tab->window_title);
 }
 
-const char *multi_tab_get_name_or_title(MultiTab * tab)
+const char *multi_tab_get_name(MultiTab * tab)
 {
-    return tab->name ? tab->name : tab->title;
+    return tab->name;
+}
+
+const char *multi_tab_get_window_title(MultiTab * tab)
+{
+    return tab->window_title;
+}
+
+const char *multi_tab_get_icon_title(MultiTab * tab)
+{
+    return tab->icon_title;
+}
+
+static const char *multi_tab_get_name_or_title(MultiTab * tab)
+{
+    return tab->name ? tab->name : tab->window_title;
 }
 
 gpointer multi_tab_get_user_data(MultiTab * tab)
@@ -436,15 +491,8 @@ inline static void multi_win_shade_for_next_and_previous_tab(MultiWin * win)
     multi_win_shade_for_next_tab(win);
 }
 
-inline static void multi_win_shade_for_ntabs(MultiWin * win)
-{
-    multi_win_shade_item_in_both_menus(win, MENUTREE_FILE_CLOSE_TAB,
-        win->ntabs == 1);
-}
-
 inline static void multi_win_shade_menus_for_tabs(MultiWin * win)
 {
-    multi_win_shade_for_ntabs(win);
     multi_win_shade_for_next_and_previous_tab(win);
 }
 
@@ -636,28 +684,50 @@ static void multi_win_set_geometry_hints_for_tab(MultiWin * win, MultiTab * tab)
     }
 }
 
-static void multi_win_set_title(MultiWin *win, const char *title)
+static char *make_title(MultiWin *win, const char *title)
 {
     char *title0 = NULL;
     
+    if (win->title_template)
+    {
+        if (strstr(win->title_template, "%s"))
+        {
+            title0 = g_strdup_printf(win->title_template, title ? title: "");
+        }
+        else
+        {
+            title0 = g_strdup(win->title_template);
+        }
+    }
+    return title0;
+}
+
+static void multi_win_set_title(MultiWin *win, const char *title)
+{
+    char *title0;
+    
     if (win->child_title != title)
     {
         g_free(win->child_title);
         win->child_title = title ? g_strdup(title) : NULL;
     }
-    if (win->title_template)
+    title0 = make_title(win, title);
+    gtk_window_set_title(GTK_WINDOW(win->gtkwin), title0);
+    g_free(title0);
+}
+
+static void multi_win_set_icon_title(MultiWin *win, const char *title)
+{
+    char *title0;
+    
+    if (win->child_title != title)
     {
-        if (strstr(win->title_template, "%s"))
-        {
-            title0 = g_strdup_printf(win->title_template, title);
-            title = title0;
-        }
-        else
-        {
-            title = win->title_template;
-        }
+        g_free(win->child_title);
+        win->child_title = title ? g_strdup(title) : NULL;
     }
-    gtk_window_set_title(GTK_WINDOW(win->gtkwin), title);
+    title0 = make_title(win, title);
+    if (win->gtkwin->window)
+        gdk_window_set_icon_name(win->gtkwin->window, title0);
     g_free(title0);
 }
 
@@ -674,17 +744,19 @@ void multi_win_select_tab(MultiWin * win, MultiTab * tab)
     if (tab)
     {
         multi_tab_cancel_attention(tab);
+        multi_tab_set_status_stock(tab, NULL);
         win->user_data_template = tab->user_data;
         gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook),
             multi_tab_get_page_num(tab));
         gtk_widget_grab_focus(tab->active_widget);
-        multi_win_set_title(win, tab->title);
+        multi_win_set_title(win, tab->window_title);
+        multi_win_set_icon_title(win, tab->icon_title);
         menutree_select_tab(win->popup_menu, tab->popup_menu_item);
         menutree_select_tab(win->menu_bar, tab->menu_bar_item);
         if (GTK_WIDGET_REALIZED(tab->active_widget) &&
             win->tab_selection_handler)
         {
-            (*win->tab_selection_handler) (tab->user_data, tab);
+            (*win->tab_selection_handler)(tab->user_data, tab);
         }
     }
     multi_win_shade_for_next_and_previous_tab(win);
@@ -741,7 +813,12 @@ void multi_win_set_show_menu_bar(MultiWin * win, gboolean show, gboolean resize)
     win->ignore_size_allocate = 0;
 }
 
-static void multi_win_set_fullscreen(MultiWin *win, gboolean fullscreen)
+gboolean multi_win_get_show_menu_bar(MultiWin * win)
+{
+    return win->show_menu_bar;
+}
+
+void multi_win_set_fullscreen(MultiWin *win, gboolean fullscreen)
 {
     if (fullscreen == win->fullscreen)
         return;
@@ -753,6 +830,7 @@ static void multi_win_set_fullscreen(MultiWin *win, gboolean fullscreen)
     {
         gtk_window_unfullscreen(GTK_WINDOW(win->gtkwin));
     }
+    win->fullscreen = fullscreen;
 }
 
 static gboolean multi_win_state_event_handler(GtkWidget *widget,
@@ -859,6 +937,7 @@ static gboolean multi_win_focus_in(GtkWidget *widget, GdkEventFocus *event,
 {
     multi_win_all = g_list_remove(multi_win_all, win);
     multi_win_all = g_list_prepend(multi_win_all, win);
+    gtk_window_set_urgency_hint(GTK_WINDOW(win->gtkwin), FALSE);
     return FALSE;
 }
 
@@ -955,6 +1034,7 @@ static GtkNotebook *multi_win_notebook_creation_hook(GtkNotebook *source,
      * signals after leaving this function; see notes in above handlers */
     MultiTab *tab = multi_tab_from_widget(page);
     MultiWin *win;
+    gboolean disable_menu_shortcuts, disable_tab_shortcuts;
 
     g_return_val_if_fail(tab, NULL);
     win = tab->parent;
@@ -963,9 +1043,12 @@ static GtkNotebook *multi_win_notebook_creation_hook(GtkNotebook *source,
         MultiWin_GetTargetSize,
         &win->child_width, &win->child_height);
     multi_tab_remove_from_parent(tab, TRUE);
+    multi_win_get_disable_menu_shortcuts(tab->user_data,
+            &disable_menu_shortcuts, &disable_tab_shortcuts);
     win = multi_win_new_blank(win->display_name,
                 multi_win_get_shortcut_scheme(win),
-                ((MultiWin *) data)->zoom_index, multi_tab_get_user_data(tab),
+                ((MultiWin *) data)->zoom_index,
+                disable_menu_shortcuts, disable_tab_shortcuts,
                 win->tab_pos, win->always_show_tabs);
     multi_win_add_tab(win, tab, -1, TRUE);
     gtk_widget_show_all(win->notebook);
@@ -981,16 +1064,33 @@ static void multi_win_close_tab_clicked(GtkWidget *widget, MultiTab *tab)
     multi_tab_delete(tab);
 }
 
-static void multi_win_new_window_action(MultiWin * win)
+MultiWin *multi_win_clone(MultiWin *old,
+        gpointer user_data_template, gboolean always_show_tabs)
 {
     GtkPositionType tab_pos = GTK_POS_TOP;
     int num_tabs = 1;
+    MultiWin *(*creator)(const char *display_name,
+        Options *shortcuts, int zoom_index,
+        gpointer user_data_template, int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs);
 
-    multi_win_initial_tabs(win->current_tab->user_data, &tab_pos, &num_tabs);
-    multi_win_new(win->display_name, win->shortcuts, win->zoom_index,
-              win->current_tab ? win->current_tab->user_data :
-              win->user_data_template, num_tabs, tab_pos,
-              win->always_show_tabs);
+    multi_win_initial_tabs(user_data_template, &tab_pos, &num_tabs);
+    if (old->fullscreen)
+        creator = multi_win_new_fullscreen;
+    else if (multi_win_is_maximised(old))
+        creator = multi_win_new_maximised;
+    else
+        creator = multi_win_new;
+    return creator(old->display_name, old->shortcuts, old->zoom_index,
+            user_data_template, num_tabs, tab_pos, always_show_tabs);
+}
+
+static void multi_win_new_window_action(MultiWin * win)
+{
+    multi_win_clone(win,
+            win->current_tab ?
+                    win->current_tab->user_data : win->user_data_template,
+            win->always_show_tabs);
 }
 
 static void multi_win_new_tab_action(MultiWin * win)
@@ -1018,20 +1118,34 @@ static void multi_win_name_tab_action(MultiWin * win)
     GtkWidget *name_w = gtk_entry_new();
     GtkEntry *name_e = GTK_ENTRY(name_w);
     const char *name = tab->name;
+    int response;
 
+    tab->rename_dialog = dialog_w;
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_box_pack_start(GTK_BOX(dialog->vbox), name_w, TRUE, TRUE, 8);
     gtk_entry_set_activates_default(name_e, TRUE);
     gtk_entry_set_text(name_e, name ? name : "");
     gtk_widget_show_all(dialog_w);
-    if (gtk_dialog_run(dialog) == GTK_RESPONSE_APPLY)
+    response = gtk_dialog_run(dialog);
+    switch (response)
     {
-        name = gtk_entry_get_text(name_e);
-        if (name && !name[0])
-            name = NULL;
-        multi_tab_set_name(tab, name);
+        case GTK_RESPONSE_NONE:
+        case GTK_RESPONSE_DELETE_EVENT:
+            /* Don't destroy */
+            break;
+        case GTK_RESPONSE_APPLY:
+            name = gtk_entry_get_text(name_e);
+            if (name && !name[0])
+                name = NULL;
+            multi_tab_set_name(tab, name);
+            /* Fall through to destroy */
+        default:
+            gtk_widget_destroy(dialog_w);
+            break;
     }
-    gtk_widget_destroy(dialog_w);
+    tab->rename_dialog = NULL;
+    if (tab->postponed_free)
+        g_free(tab);
 }
 
 static void multi_win_set_window_title_action(MultiWin * win)
@@ -1057,6 +1171,7 @@ static void multi_win_set_window_title_action(MultiWin * win)
             "characters or sequences are permitted except '%%' which is "
             "displayed as a single %. Apply an empty string here to use the "
             "profile's title string."));
+    int response;
 
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_box_pack_start(GTK_BOX(dialog->vbox), title_w, FALSE, TRUE, 8);
@@ -1067,15 +1182,24 @@ static void multi_win_set_window_title_action(MultiWin * win)
     gtk_entry_set_activates_default(title_e, TRUE);
     gtk_entry_set_text(title_e, title ? title : "");
     gtk_widget_show_all(dialog_w);
-    if (gtk_dialog_run(dialog) == GTK_RESPONSE_APPLY)
+    response = gtk_dialog_run(dialog);
+    switch (response)
     {
-        title = gtk_entry_get_text(title_e);
-        if (title && !title[0])
-            title = NULL;
-        multi_win_set_title_template(win, title, FALSE);
-        win->title_override = title ? TRUE : FALSE;
+        case GTK_RESPONSE_DELETE_EVENT:
+        case GTK_RESPONSE_NONE:
+            /* Don't destroy */
+            break;
+        case GTK_RESPONSE_APPLY:
+            title = gtk_entry_get_text(title_e);
+            if (title && !title[0])
+                title = NULL;
+            multi_win_set_title_template(win, title, FALSE);
+            win->title_override = title ? TRUE : FALSE;
+            /* Fall through to destroy */
+        default:
+            gtk_widget_destroy(dialog_w);
+            break;
     }
-    gtk_widget_destroy(dialog_w);
 }
 
 static void multi_win_next_tab_action(MultiWin * win)
@@ -1110,6 +1234,15 @@ static void multi_win_select_tab_action(GtkCheckMenuItem *widget,
         multi_win_select_tab(win, tab);
 }
 
+static void
+multi_win_toggle_show_tab_bar_action(GtkCheckMenuItem * item, MultiWin * win)
+{
+    gboolean show = gtk_check_menu_item_get_active(item);
+
+    if (show != win->always_show_tabs)
+        multi_win_set_always_show_tabs(win, show);
+}
+
 static void
 multi_win_toggle_show_menubar_action(GtkCheckMenuItem * item, MultiWin * win)
 {
@@ -1244,6 +1377,8 @@ static void multi_win_connect_actions(MultiWin * win)
         (multi_win_name_tab_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_EDIT_SET_WINDOW_TITLE,
         G_CALLBACK(multi_win_set_window_title_action), win, NULL, NULL, NULL);
+    multi_win_menu_connect(win, MENUTREE_VIEW_SHOW_TAB_BAR, G_CALLBACK
+        (multi_win_toggle_show_tab_bar_action), win, NULL, NULL, NULL);
     multi_win_menu_connect(win, MENUTREE_VIEW_SHOW_MENUBAR, G_CALLBACK
         (multi_win_toggle_show_menubar_action), win, NULL, NULL, NULL);
     multi_win_menu_connect(win, MENUTREE_VIEW_FULLSCREEN, G_CALLBACK
@@ -1260,6 +1395,12 @@ static void multi_win_connect_actions(MultiWin * win)
         (multi_win_scroll_down_action), win, NULL, NULL, NULL);
 }
 
+static void multi_win_set_show_tabs_menu_items(MultiWin *win, gboolean active)
+{
+    menutree_set_show_tab_bar_active(win->menu_bar, active);
+    menutree_set_show_tab_bar_active(win->popup_menu, active);
+}
+
 static void multi_win_show_tabs(MultiWin * win)
 {
     if (win->tab_pos != -1)
@@ -1284,14 +1425,11 @@ void multi_win_show(MultiWin *win)
 }
 
 MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
-        int zoom_index, gpointer user_data_template, GtkPositionType tab_pos,
-        gboolean always_show_tabs)
+        int zoom_index,
+        gboolean disable_menu_shortcuts, gboolean disable_tab_shortcuts,
+        GtkPositionType tab_pos, gboolean always_show_tabs)
 {
     MultiWin *win = g_new0(MultiWin, 1);
-    gboolean disable_menu_shortcuts, disable_tab_menu_shortcuts;
-    
-    multi_win_get_disable_menu_shortcuts(user_data_template,
-            &disable_menu_shortcuts, &disable_tab_menu_shortcuts);
 
 #if !DO_OWN_TAB_DRAGGING
     static gboolean set_nwc_hook = FALSE;
@@ -1307,9 +1445,10 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
     win->tab_pos = tab_pos;
     win->always_show_tabs = always_show_tabs;
     win->scroll_bar_pos = MultiWinScrollBar_Query;
-    win->user_data_template = user_data_template;
     if (zoom_index < 0)
         win->zoom_index = MULTI_WIN_NORMAL_ZOOM_INDEX;
+    else if (zoom_index >= MULTI_WIN_N_ZOOM_FACTORS)
+        win->zoom_index = multi_win_zoom_factors[MULTI_WIN_N_ZOOM_FACTORS - 1];
     else
         win->zoom_index = zoom_index;
 
@@ -1323,6 +1462,14 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         if (scr)
             gtk_window_set_screen(GTK_WINDOW(win->gtkwin), scr);
     }
+    if (multi_win_role_prefix)
+    {
+        char *role = g_strdup_printf("%s-%d-%x-%d", multi_win_role_prefix,
+                getpid(), g_random_int(), ++multi_win_role_index);
+        
+        gtk_window_set_role(GTK_WINDOW(win->gtkwin), role);
+        g_free(role);
+    }
     g_signal_connect(win->gtkwin, "realize",
             G_CALLBACK(multi_win_realize_handler), win);
     win->destroy_handler = g_signal_connect(win->gtkwin, "destroy",
@@ -1358,7 +1505,7 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
     gtk_window_add_accel_group(GTK_WINDOW(win->gtkwin), win->accel_group);
 
     win->popup_menu = menutree_new(shortcuts, win->accel_group,
-        GTK_TYPE_MENU, disable_menu_shortcuts, disable_tab_menu_shortcuts, win);
+        GTK_TYPE_MENU, disable_menu_shortcuts, disable_tab_shortcuts, win);
     menutree_connect_destroyed(win->popup_menu,
         G_CALLBACK(multi_win_menutree_deleted_handler), win);
 
@@ -1368,7 +1515,7 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         G_CALLBACK(multi_win_menutree_deleted_handler), win);
 
     win->menu_bar = menutree_new(shortcuts, win->accel_group,
-        GTK_TYPE_MENU_BAR, disable_menu_shortcuts, disable_tab_menu_shortcuts,
+        GTK_TYPE_MENU_BAR, disable_menu_shortcuts, disable_tab_shortcuts,
         win);
     /* Make sure menu widgets aren't destroyed when removed from window */
     g_object_ref(menutree_get_top_level_widget(win->menu_bar));
@@ -1390,11 +1537,25 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         G_CALLBACK(multi_win_state_event_handler), win);
 
     win->notebook = gtk_notebook_new();
+    if (win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT)
+    {
+        g_object_set(win->notebook, "tab-hborder", 0, NULL);
+    }
+    else
+    {
+        g_object_set(win->notebook, "tab-vborder", 0, NULL);
+    }
     gtk_notebook_set_scrollable(GTK_NOTEBOOK(win->notebook), TRUE);
     if (always_show_tabs)
+    {
+        multi_win_set_show_tabs_menu_items(win, TRUE);
         multi_win_show_tabs(win);
+    }
     else
+    {
+        multi_win_set_show_tabs_menu_items(win, FALSE);
         multi_win_hide_tabs(win);
+    }
     gtk_box_pack_start_defaults(GTK_BOX(win->vbox), win->notebook);
     g_signal_connect(win->notebook, "switch-page",
         G_CALLBACK(multi_win_page_switched), win);
@@ -1418,16 +1579,28 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
     return win;
 }
 
+void multi_win_set_role_prefix(const char *role_prefix)
+{
+    g_free(multi_win_role_prefix);
+    multi_win_role_prefix = g_strdup(role_prefix);
+}
+
 MultiWin *multi_win_new_full(const char *display_name, Options *shortcuts,
         int zoom_index, gpointer user_data_template, const char *geom,
-        gboolean fullscreen, int numtabs, GtkPositionType tab_pos,
+        MultiWinSizing sizing, int numtabs, GtkPositionType tab_pos,
         gboolean always_show_tabs)
 {
-    MultiWin *win = multi_win_new_blank(display_name, shortcuts, zoom_index,
-            user_data_template, tab_pos, always_show_tabs);
-
+    gboolean disable_menu_shortcuts, disable_tab_shortcuts; 
+    MultiWin *win;
+    
+    multi_win_get_disable_menu_shortcuts(user_data_template,
+            &disable_menu_shortcuts, &disable_tab_shortcuts);
+    win = multi_win_new_blank(display_name, shortcuts, zoom_index,
+            disable_menu_shortcuts, disable_tab_shortcuts,
+            tab_pos, always_show_tabs);
+    win->user_data_template = user_data_template;
     win->tab_pos = tab_pos;
-    while(numtabs--)
+    while (numtabs--)
         multi_tab_new(win, user_data_template);
 
     gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook), 0);
@@ -1448,10 +1621,14 @@ MultiWin *multi_win_new_full(const char *display_name, Options *shortcuts,
     {
         gtk_window_parse_geometry(GTK_WINDOW(win->gtkwin), geom);
     }
-    else if (fullscreen)
+    else if (sizing == MULTI_WIN_FULL_SCREEN)
     {
         multi_win_set_fullscreen(win, TRUE);
     }
+    else if (sizing == MULTI_WIN_MAXIMISED)
+    {
+        gtk_window_maximize(GTK_WINDOW(win->gtkwin));
+    }
     multi_win_show(win);
 
     /* Need to do this after multi_tab_new's initial call of
@@ -1514,6 +1691,16 @@ void multi_win_delete(MultiWin *win)
     multi_win_destructor(win, TRUE);
 }
 
+static void single_tab(MultiWin *win, MultiTab *tab)
+{
+    gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(win->notebook),
+        tab->widget, FALSE, TRUE, GTK_PACK_START);
+    gtk_widget_set_size_request(tab->label,
+            (win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT) ?
+                    HORIZ_TAB_WIDTH : -1,
+            -1);
+}
+
 /* Returns TRUE if window destroyed */
 static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
 {
@@ -1537,9 +1724,17 @@ static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
     }
     else
     {
-        if (win->ntabs == 1 && !win->always_show_tabs)
+        if (win->ntabs == 1)
         {
-            multi_win_hide_tabs(win);
+            tab = win->tabs->data;
+            if (!win->always_show_tabs)
+                multi_win_hide_tabs(win);
+            if (win->tab_pos != GTK_POS_LEFT && win->tab_pos != GTK_POS_RIGHT)
+            {
+                gtk_label_set_ellipsize(GTK_LABEL(tab->label),
+                        PANGO_ELLIPSIZE_NONE);
+            }
+            single_tab(win, tab);
         }
         if (win->current_tab)
         {
@@ -1554,7 +1749,6 @@ static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
 
 void multi_tab_add_close_button(MultiTab *tab)
 {
-    GtkWidget *image;
     GtkRcStyle *rcstyle;
     GtkSettings *settings;
     int w, h;
@@ -1565,16 +1759,22 @@ void multi_tab_add_close_button(MultiTab *tab)
     
     win = tab->parent;
     if (win)
-        win->ignore_size_allocate = 1;
+    {
+        win->ignore_size_allocate =
+                (win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT)
+                ? 2 : 1;
+    }
     tab->close_button = gtk_button_new();
-    image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+    tab->image = gtk_image_new_from_stock(
+            tab->status_stock ? tab->status_stock : GTK_STOCK_CLOSE,
+            GTK_ICON_SIZE_MENU);
     rcstyle = gtk_rc_style_new();
-    gtk_container_add(GTK_CONTAINER(tab->close_button), image);
+    gtk_container_add(GTK_CONTAINER(tab->close_button), tab->image);
     gtk_button_set_relief(GTK_BUTTON(tab->close_button), GTK_RELIEF_NONE);
     gtk_button_set_focus_on_click(GTK_BUTTON(tab->close_button), FALSE);
     settings = gtk_widget_get_settings(tab->label);
     gtk_icon_size_lookup_for_settings(settings, GTK_ICON_SIZE_MENU, &w, &h);
-    gtk_widget_set_size_request(tab->close_button, w + 2, h + 2);
+    /* gtk_widget_set_size_request(tab->close_button, w + 2, h + 2); */
     rcstyle->xthickness = rcstyle->ythickness = 0;
     gtk_widget_modify_style(tab->close_button, rcstyle);
     gtk_rc_style_unref(rcstyle);
@@ -1592,7 +1792,8 @@ void multi_tab_add_close_button(MultiTab *tab)
         }
         else
         {
-            win->ignore_size_allocate = 0;
+            if (win->ignore_size_allocate == 1)
+                win->ignore_size_allocate = 0;
         }
     }
 }
@@ -1605,7 +1806,12 @@ void multi_tab_remove_close_button(MultiTab *tab)
     
         win = tab->parent;
         if (win)
-            win->ignore_size_allocate = 1;
+        {
+            win->ignore_size_allocate =
+                    (win->tab_pos == GTK_POS_LEFT ||
+                            win->tab_pos == GTK_POS_RIGHT)
+                    ? 2 : 1;
+        }
         gtk_widget_destroy(tab->close_button);
         tab->close_button = NULL;
         if (win)
@@ -1617,15 +1823,33 @@ void multi_tab_remove_close_button(MultiTab *tab)
             }
             else
             {
-                win->ignore_size_allocate = 0;
+                if (win->ignore_size_allocate == 1)
+                    win->ignore_size_allocate = 0;
             }
         }
     }
+    tab->image = NULL;
+}
+
+void multi_tab_set_status_stock(MultiTab *tab, const char *stock)
+{
+    if (!strcmp(tab->status_stock ? tab->status_stock : GTK_STOCK_CLOSE,
+            stock ? stock : GTK_STOCK_CLOSE))
+    {
+        return;
+    }
+    tab->status_stock = stock;
+    if (tab->image)
+    {
+        gtk_image_set_from_stock(GTK_IMAGE(tab->image),
+                stock ? stock : GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+    }
 }
 
 /* Creates the label widget for a tab. tab->label is the GtkLabel containing
  * the text; the return value is the top-level container. */
-static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
+static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos,
+        gboolean single)
 {
     static GdkColor amber;
     static gboolean parsed_amber = FALSE;
@@ -1637,16 +1861,16 @@ static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
         case GTK_POS_RIGHT:
             tab->label_box = gtk_vbox_new(FALSE, 4);
             tab->label_packer = gtk_box_pack_end;
-            gtk_label_set_ellipsize(GTK_LABEL(tab->label),
-                    PANGO_ELLIPSIZE_NONE);
+            gtk_widget_set_size_request(tab->label, HORIZ_TAB_WIDTH, -1);
+            single = FALSE;
             break;
         default:
             tab->label_box = gtk_hbox_new(FALSE, 4);
             tab->label_packer = gtk_box_pack_start;
-            gtk_label_set_ellipsize(GTK_LABEL(tab->label),
-                    PANGO_ELLIPSIZE_MIDDLE);
-             break;
+            break;
     }
+    if (!single)
+        gtk_label_set_ellipsize(GTK_LABEL(tab->label), PANGO_ELLIPSIZE_MIDDLE);
     
     tab->label_bg = gtk_event_box_new();
     if (!parsed_amber)
@@ -1666,13 +1890,8 @@ static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
 #endif
     if (multi_tab_get_show_close_button(tab->user_data))
     {
-        g_debug("Adding close button");
         multi_tab_add_close_button(tab);
     }
-    else
-    {
-        g_debug("Not adding close button");
-    }
     gtk_notebook_set_tab_label(GTK_NOTEBOOK(tab->parent->notebook),
         tab->widget, tab->label_box);
     gtk_widget_show_all(tab->label_box);
@@ -1701,8 +1920,15 @@ static void multi_tab_add_menutree_items(MultiWin * win, MultiTab * tab,
 static void multi_win_add_tab_to_notebook(MultiWin * win, MultiTab * tab,
         int position)
 {
-    GtkWidget *label = make_tab_label(tab, win->tab_pos);
+    GtkWidget *label = make_tab_label(tab, win->tab_pos, win->ntabs == 1);
 
+    if (win->ntabs == 2)
+    {
+        gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(win->notebook),
+            win->current_tab->widget, TRUE, TRUE, GTK_PACK_START);
+        gtk_label_set_ellipsize(GTK_LABEL(win->current_tab->label),
+                PANGO_ELLIPSIZE_MIDDLE);
+    }
     if (position == -1)
     {
         gtk_notebook_append_page(GTK_NOTEBOOK(win->notebook),
@@ -1720,8 +1946,15 @@ static void multi_win_add_tab_to_notebook(MultiWin * win, MultiTab * tab,
             TRUE);
 #endif
     g_object_set(gtk_widget_get_parent(label), "can-focus", FALSE, NULL);
-    gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(win->notebook),
-        tab->widget, TRUE, TRUE, GTK_PACK_START);
+    if (win->ntabs == 1)
+    {
+        single_tab(win, tab);
+    }
+    else
+    {
+        gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(win->notebook),
+            tab->widget, TRUE, TRUE, GTK_PACK_START);
+    }
 }
 
 /* Keep track of a tab after it's been created; if notify_only is set, it's
@@ -1759,7 +1992,6 @@ static void multi_win_add_tab(MultiWin * win, MultiTab * tab, int position,
     multi_win_set_size(win, tab->active_widget,
         win->child_width, win->child_height);
     win->ignore_tabs_moving = FALSE;
-    win->ignore_size_allocate = 0;
 }
 
 GtkWidget *multi_win_get_widget(MultiWin * win)
@@ -1772,6 +2004,12 @@ gboolean multi_win_is_fullscreen(MultiWin *win)
     return win->fullscreen;
 }
 
+gboolean multi_win_is_maximised(MultiWin *win)
+{
+    return (gdk_window_get_state(win->gtkwin->window) &
+            GDK_WINDOW_STATE_MAXIMIZED) != 0;
+}
+
 int multi_win_get_zoom_index(MultiWin *win)
 {
     return win->zoom_index;
@@ -1953,6 +2191,7 @@ void multi_win_set_always_show_tabs(MultiWin *win, gboolean show)
     gboolean old_show = win->always_show_tabs;
     
     win->always_show_tabs = show;
+    multi_win_set_show_tabs_menu_items(win, show);
     if (win->ntabs == 1 && old_show != show)
     {
         MultiTab *tab = win->current_tab;
@@ -2041,11 +2280,16 @@ void multi_win_set_title_template(MultiWin *win, const char *tt, gboolean def)
     {
         g_free(win->title_template);
         win->title_template = tt ? g_strdup(tt) : NULL;
-        if (win->child_title)
-            multi_win_set_title(win, win->child_title);
+    	multi_win_set_title(win, win->child_title);
+	multi_win_set_icon_title(win, win->child_icon);
     }
 }
 
+const char *multi_win_get_title_template(MultiWin *win)
+{
+    return win->title_template;
+}
+
 gboolean multi_win_composite(MultiWin *win)
 {
 #if HAVE_COMPOSITE
@@ -2060,7 +2304,19 @@ const char *multi_win_get_display_name(MultiWin *win)
     return win->display_name;
 }
 
+const char *multi_win_get_shortcuts_scheme_name(MultiWin *win)
+{
+    char *slash = strrchr(win->shortcuts->name, G_DIR_SEPARATOR);
+    
+    return slash ? slash + 1 : win->shortcuts->name;
+}
+
+guint multi_win_get_num_tabs(MultiWin *win)
+{
+    return g_list_length(win->tabs);
+}
+
 /* This is maintained in order of most recently focused */
 GList *multi_win_all = NULL;
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
