@@ -63,7 +63,7 @@ gboolean options_copy_keyfile(Options *dest, const Options *src)
 	if (err)
 		goto copy_err;
 	dest->kf = g_key_file_new();
-	if (g_key_file_load_from_data(dest->kf, kf_data, l,
+	if (!l || g_key_file_load_from_data(dest->kf, kf_data, l,
 		G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, &err))
 	{
 		goto copy_done;
