@@ -32,36 +32,51 @@
 #include "options.h"
 #include "profilegui.h"
 
-#define PG_CONNECT(pg, h) \
-    glade_xml_signal_connect_data ((pg)->glade, #h, (GCallback) (h), (pg))
+#define MANAGE_PREVIEW
 
-#define PG_CONNECT_OPTS(pg, h) \
-    glade_xml_signal_connect_data ((pg)->glade, #h, (GCallback) (h), \
+#define PG_GLADE_CONNECT(pg, glade, h) \
+    glade_xml_signal_connect_data ((glade), #h, (GCallback) (h), (pg))
+
+#define PG_CONNECT(pg, h) PG_GLADE_CONNECT((pg), (pg)->glade, h)
+
+#define PG_GLADE_CONNECT_OPTS(pg, glade, h) \
+    glade_xml_signal_connect_data ((glade), #h, (GCallback) (h), \
             (pg)->profile)
 
-#define PG_EC_CONNECT(pg, n) \
-    glade_xml_signal_connect_data ((pg)->glade, "on_"  #n "_changed", \
+#define PG_CONNECT_OPTS(pg, h) PG_GLADE_CONNECT_OPTS((pg), (pg)->glade, h)
+
+#define PG_GLADE_EC_CONNECT(pg, glade, n) \
+    glade_xml_signal_connect_data ((glade), "on_"  #n "_changed", \
             (GCallback) on_editable_changed, &pg->changed. n)
 
+#define PG_EC_CONNECT(pg, n) PG_GLADE_EC_CONNECT((pg), (pg)->glade, n)
+
 struct ProfileGUI {
     GladeXML *glade;
+    GladeXML *ssh_glade;
     GtkWidget *widget;
     char *profile_name;
     Options *profile;
     gboolean ignore_destroy;
     struct {
         char sel_by_word;
+        char color_term;
+        char term;
         /*char emulation;*/
         char browser;
         char mailer;
         char command;
         char title_string;
+        char ssh_address;
+        char ssh_user;
+        char ssh_options;
     } changed;
     GtkWidget *bgimg;
 #ifndef HAVE_VTE_TERMINAL_SET_CURSOR_BLINK_MODE
     GtkWidget *cursor_blinks;
 #endif
     DragReceiveData *bgimg_drd;
+    GtkListStore *list_store;
 };
 
 static GHashTable *profilegui_being_edited = NULL;
@@ -72,10 +87,17 @@ GtkWidget *profilegui_widget(ProfileGUI *pg, const char *name)
     return glade_xml_get_widget(pg->glade, name);
 }
 
-static void profilegui_update_from_entry(ProfileGUI * pg, const char *name)
+static void update_from_entry(ProfileGUI * pg, GladeXML *glade,
+        const char *name)
 {
     capplet_set_string(pg->profile, name, gtk_entry_get_text
-        (GTK_ENTRY(profilegui_widget(pg, name))));
+        (GTK_ENTRY(glade_xml_get_widget(glade, name))));
+}
+
+inline static void
+profilegui_update_from_entry(ProfileGUI *pg, const char *name)
+{
+    update_from_entry(pg, pg->glade, name);
 }
 
 void profilegui_check_entries_for_changes(ProfileGUI * pg)
@@ -85,6 +107,16 @@ void profilegui_check_entries_for_changes(ProfileGUI * pg)
         profilegui_update_from_entry(pg, "sel_by_word");
         pg->changed.sel_by_word = 0;
     }
+    if (pg->changed.color_term)
+    {
+        profilegui_update_from_entry(pg, "color_term");
+        pg->changed.color_term = 0;
+    }
+    if (pg->changed.term)
+    {
+        profilegui_update_from_entry(pg, "term");
+        pg->changed.term = 0;
+    }
     /*
     if (pg->changed.emulation)
     {
@@ -150,14 +182,88 @@ static void profilegui_set_command_shading(ProfileGUI *pg)
 {
     gboolean use_custom = gtk_toggle_button_get_active(
             GTK_TOGGLE_BUTTON(profilegui_widget(pg, "use_custom_command")));
+    gboolean use_ssh = gtk_toggle_button_get_active(
+            GTK_TOGGLE_BUTTON(profilegui_widget(pg, "use_ssh")));
 
     gtk_widget_set_sensitive(profilegui_widget(pg, "command"), use_custom);
-    gtk_widget_set_sensitive(profilegui_widget(pg, "login_shell"), !use_custom);
+    gtk_widget_set_sensitive(profilegui_widget(pg, "ssh_host"), use_ssh);
+    gtk_widget_set_sensitive(profilegui_widget(pg, "edit_ssh"), use_ssh);
+    gtk_widget_set_sensitive(profilegui_widget(pg, "login_shell"),
+            !use_custom && !use_ssh);
+}
+
+static void profilegui_set_close_buttons_shading(ProfileGUI *pg)
+{
+    gtk_widget_set_sensitive(profilegui_widget(pg, "show_tab_status"),
+            gtk_toggle_button_get_active(
+            GTK_TOGGLE_BUTTON(profilegui_widget(pg, "tab_close_btn"))));
+}
+
+static void profilegui_check_ssh_entries_for_changes(ProfileGUI * pg)
+{
+    GladeXML *glade = pg->ssh_glade;
+    
+    if (pg->changed.ssh_address)
+    {
+        update_from_entry(pg, glade, "ssh_address");
+        pg->changed.ssh_address = 0;
+    }
+    if (pg->changed.ssh_user)
+    {
+        update_from_entry(pg, glade, "ssh_user");
+        pg->changed.ssh_user = 0;
+    }
+    if (pg->changed.ssh_options)
+    {
+        update_from_entry(pg, glade, "ssh_options");
+        pg->changed.ssh_options = 0;
+    }
 }
 
 /***********************************************************/
 /* Handlers */
 
+static void on_ssh_entry_activate(GtkEntry * entry, ProfileGUI * pg)
+{
+    profilegui_check_ssh_entries_for_changes(pg);
+}
+
+static void on_ssh_host_changed(GtkWidget * widget, ProfileGUI *pg)
+{
+    gtk_entry_set_text(GTK_ENTRY(profilegui_widget(pg, "ssh_host")),
+            gtk_entry_get_text(GTK_ENTRY(widget)));
+}
+
+static void on_edit_ssh_clicked(GtkWidget * widget, ProfileGUI *pg)
+{
+    GladeXML *glade = glade_xml_new(capplet_get_glade_filename(),
+        "ssh_dialog", NULL);
+    GtkWidget *dialog = glade_xml_get_widget(glade, "ssh_dialog");
+    
+    pg->ssh_glade = glade;
+    capplet_inc_windows();
+    
+    PG_GLADE_EC_CONNECT(pg, glade, ssh_address);
+    PG_GLADE_EC_CONNECT(pg, glade, ssh_user);
+    PG_GLADE_EC_CONNECT(pg, glade, ssh_options);
+    PG_GLADE_CONNECT(pg, glade, on_ssh_entry_activate);
+    PG_GLADE_CONNECT(pg, glade, on_ssh_host_changed);
+    PG_GLADE_CONNECT_OPTS(pg, glade, on_spin_button_changed);
+
+    capplet_set_text_entry(glade, pg->profile, "ssh_address", "localhost");
+    capplet_set_text_entry(glade, pg->profile, "ssh_user", NULL);
+    capplet_set_spin_button(glade, pg->profile, "ssh_port", 22);
+    capplet_set_text_entry(glade, pg->profile, "ssh_options", NULL);
+    
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    
+    profilegui_check_ssh_entries_for_changes(pg);
+    gtk_widget_destroy(dialog);
+    UNREF_LOG(g_object_unref(glade));
+    pg->ssh_glade = NULL;
+    capplet_dec_windows();
+}
+
 static void on_Profile_Editor_destroy(GtkWidget * widget, ProfileGUI * pg)
 {
     if (pg->ignore_destroy)
@@ -207,10 +313,14 @@ static void on_bgtype_toggled(GtkToggleButton *button, ProfileGUI *pg)
     on_radio_toggled(button, pg->profile);
 }
 
-static void on_custom_command_toggled(GtkToggleButton *button, ProfileGUI *pg)
+static void on_command_toggled(GtkToggleButton *button, ProfileGUI *pg)
 {
     profilegui_set_command_shading(pg);
-    on_boolean_toggled(button, pg->profile);
+}
+
+static void on_close_buttons_toggled(GtkToggleButton *button, ProfileGUI *pg)
+{
+    profilegui_set_close_buttons_shading(pg);
 }
 
 #ifndef HAVE_VTE_TERMINAL_SET_CURSOR_BLINK_MODE
@@ -234,6 +344,7 @@ inline static gboolean pg_image_reportable(ProfileGUI *pg)
                 profilegui_widget(pg, "background_type1")) == 1);
 }
 
+#ifdef MANAGE_PREVIEW
 static void
 pg_load_preview(GtkFileChooser *chooser, GtkWidget *preview,
         const char *filename, ProfileGUI *pg)
@@ -254,8 +365,15 @@ pg_load_preview(GtkFileChooser *chooser, GtkWidget *preview,
                     PREVIEW_SIZE, PREVIEW_SIZE, TRUE, &error);
         }
     }
+    else if (filename)
+    {
+        /* Get an appropriate error */
+        pixbuf = gdk_pixbuf_new_from_file_at_scale(filename,
+                PREVIEW_SIZE, PREVIEW_SIZE, TRUE, &error);
+    }
     gtk_image_set_from_pixbuf(GTK_IMAGE(preview), pixbuf);
-    if (!pixbuf && filename && filename[0])
+    if (!pixbuf && filename && filename[0] &&
+            !g_file_test(filename, G_FILE_TEST_IS_DIR))
     {
         if (pg_image_reportable(pg))
         {
@@ -292,13 +410,16 @@ static void on_update_preview(GtkFileChooser *chooser, ProfileGUI *pg)
     pg_load_preview(chooser, preview, filename, pg);
     g_free(filename);
 }
+#endif
 
 static void profilegui_setup_file_chooser(ProfileGUI *pg,
         GtkFileChooser *chooser)
 {
     GtkFileFilter *img_filter = gtk_file_filter_new();
     GtkFileFilter *all_filter = gtk_file_filter_new();
+#ifdef MANAGE_PREVIEW
     GtkWidget *preview = gtk_image_new();
+#endif
 
     gtk_file_filter_set_name(img_filter, _("Image files"));
     gtk_file_filter_set_name(all_filter, "All filetypes");
@@ -308,24 +429,35 @@ static void profilegui_setup_file_chooser(ProfileGUI *pg,
     gtk_file_chooser_add_filter(chooser, all_filter);
 
     gtk_file_chooser_set_use_preview_label(chooser, FALSE);
+#ifdef MANAGE_PREVIEW
     gtk_file_chooser_set_preview_widget(chooser, preview);
     g_signal_connect(chooser, "update-preview",
             G_CALLBACK(on_update_preview), pg);
+#endif
 }
 
 static void profilegui_fill_in_file_chooser(ProfileGUI *pg,
         GtkFileChooser *chooser, const char *old_filename)
 {
+#ifdef MANAGE_PREVIEW
     GtkWidget *preview = gtk_file_chooser_get_preview_widget(chooser);
+#endif
 
     gtk_file_chooser_set_filename(chooser, old_filename);
+#ifdef MANAGE_PREVIEW
     pg_load_preview(chooser, preview, old_filename, pg);
+#endif
 }
 
 static char *get_bgimg_filename(ProfileGUI *pg, GtkFileChooser *chooser)
 {
     char *new_filename = gtk_file_chooser_get_filename(chooser);
 
+    if (!g_file_test(new_filename, G_FILE_TEST_IS_REGULAR))
+    {
+        g_free(new_filename);
+        return NULL;
+    }
     capplet_set_string(pg->profile, "background_img", new_filename);
     return new_filename;
 }
@@ -355,8 +487,11 @@ static void on_bgimg_browse_clicked(GtkButton *button, ProfileGUI *pg)
     {
         char *new_filename = get_bgimg_filename(pg, chooser);
 
-        gtk_entry_set_text(bgimg_display, new_filename);
-        g_free(new_filename);
+        if (new_filename)
+        {
+            gtk_entry_set_text(bgimg_display, new_filename);
+            g_free(new_filename);
+        }
     }
     /*
     UNREF_LOG(g_object_unref(img_filter));
@@ -439,17 +574,13 @@ static void profilegui_connect_handlers(ProfileGUI * pg)
 
     /* Editables */
     PG_EC_CONNECT(pg, sel_by_word);
+    PG_EC_CONNECT(pg, color_term);
+    PG_EC_CONNECT(pg, term);
     /*PG_EC_CONNECT(pg, emulation);*/
     PG_EC_CONNECT(pg, command);
     PG_EC_CONNECT(pg, browser);
     PG_EC_CONNECT(pg, mailer);
     PG_EC_CONNECT(pg, title_string);
-    /*
-    glade_xml_signal_connect_data(pg->glade, "on_browser_changed",
-            (GCallback) pg_special_editable_changed, &pg->changed.browser);
-    glade_xml_signal_connect_data(pg->glade, "on_mailer_changed",
-            (GCallback) pg_special_editable_changed, &pg->changed.mailer);
-    */
 
     /* Misc */
     PG_CONNECT(pg, on_Profile_Editor_destroy);
@@ -460,10 +591,10 @@ static void profilegui_connect_handlers(ProfileGUI * pg)
     /* Unique widgets */
     PG_CONNECT(pg, on_font_set);
     PG_CONNECT(pg, on_bgtype_toggled);
-    PG_CONNECT(pg, on_custom_command_toggled);
+    PG_CONNECT(pg, on_command_toggled);
+    PG_CONNECT(pg, on_close_buttons_toggled);
 #ifdef HAVE_GTK_FILE_CHOOSER_BUTTON_NEW
-    g_signal_connect(pg->bgimg, "selection-changed",
-            G_CALLBACK(on_bgimg_chosen), pg);
+    g_signal_connect(pg->bgimg, "file-set", G_CALLBACK(on_bgimg_chosen), pg);
 #else
     PG_CONNECT(pg, on_bgimg_browse_clicked);
 #endif
@@ -474,6 +605,7 @@ static void profilegui_connect_handlers(ProfileGUI * pg)
 #endif
     g_signal_connect(profilegui_widget(pg, "exit_action"), "changed",
             G_CALLBACK(exit_action_changed), pg);
+    PG_CONNECT(pg, on_edit_ssh_clicked);
 }
 
 static void profilegui_fill_in_dialog(ProfileGUI * pg)
@@ -513,10 +645,14 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
 #endif
     capplet_set_boolean_toggle(glade, profile, "hide_mouse", TRUE);
     capplet_set_text_entry(glade, profile, "sel_by_word", "-A-Za-z0-9,./?%&#_");
+    capplet_set_text_entry(glade, profile, "color_term", "roxterm");
+    capplet_set_text_entry(glade, profile, "term", "xterm");
     /*capplet_set_text_entry(glade, profile, "emulation", NULL);*/
     capplet_set_boolean_toggle(glade, profile, "wrap_switch_tab", FALSE);
     capplet_set_boolean_toggle(glade, profile, "tab_close_btn", TRUE);
-    capplet_set_boolean_toggle(glade, profile, "always_show_tabs", FALSE);
+    capplet_set_boolean_toggle(glade, profile, "show_tab_status", FALSE);
+    capplet_set_boolean_toggle(glade, profile, "always_show_tabs", TRUE);
+    profilegui_set_close_buttons_shading(pg);
     capplet_set_text_entry(glade, profile, "browser", NULL);
     capplet_set_radio(glade, profile, "browser_spawn_type", 0);
     capplet_set_text_entry(glade, profile, "mailer", NULL);
@@ -547,6 +683,14 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_boolean_toggle(glade, profile, "disable_menu_shortcuts", FALSE);
     capplet_set_boolean_toggle(glade, profile, "disable_tab_menu_shortcuts",
             FALSE);
+    capplet_ignore_changes = TRUE;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(glade,
+                "use_default_shell")), TRUE);
+    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(glade, "ssh_host")),
+            options_lookup_string_with_default(profile, "ssh_address",
+                    "localhost"));
+    capplet_ignore_changes = FALSE;
+    capplet_set_boolean_toggle(glade, profile, "use_ssh", FALSE);
     capplet_set_boolean_toggle(glade, profile, "login_shell", FALSE);
     capplet_set_boolean_toggle(glade, profile, "update_records", TRUE);
     capplet_set_boolean_toggle(glade, profile, "use_custom_command", FALSE);
@@ -611,6 +755,57 @@ static void profilegui_replace_blink_mode_widgets(ProfileGUI *pg)
 }
 #endif
 
+static gboolean page_selected(GtkTreeSelection *selection,
+        GtkTreeModel *model, GtkTreePath *path,
+        gboolean path_currently_selected, gpointer pg)
+{
+    GtkTreeIter iter;
+    int page;
+    
+    if (!gtk_tree_model_get_iter(model, &iter, path))
+        return FALSE;
+    gtk_tree_model_get(model, &iter, 1, &page, -1);
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(profilegui_widget(pg,
+            "profile_notebook")), page);
+    return TRUE;
+}
+
+static void profilegui_setup_list_store(ProfileGUI *pg)
+{
+    static char const *labels[] = {
+            N_("Appearance"), N_("General"), N_("Command"),
+            N_("URI handling"), N_("Background"), N_("Scrolling"),
+            N_("Keyboard"), N_("Window/Tabs")
+    };
+    GtkTreeIter iter;
+    int n;
+    GtkTreeViewColumn *column;
+    GtkWidget *tvw;
+    GtkTreeView *tv;
+    GtkTreeSelection *sel;
+    
+    pg->list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+    for (n = 0; n < 8; ++n)
+    {
+        gtk_list_store_append(pg->list_store, &iter);
+        gtk_list_store_set(pg->list_store, &iter,
+                0, gettext(labels[n]), 1, n, -1);
+    }
+    tvw = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pg->list_store));
+    tv = GTK_TREE_VIEW(tvw);
+    gtk_tree_view_set_headers_visible(tv, FALSE);
+    sel = gtk_tree_view_get_selection(tv);
+    column = gtk_tree_view_column_new_with_attributes("Page",
+            gtk_cell_renderer_text_new(), "text", 0, NULL);
+    gtk_tree_view_append_column(tv, column);
+    gtk_tree_selection_set_select_function(sel, page_selected, pg, NULL);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pg->list_store), &iter);
+    gtk_tree_selection_select_iter(sel, &iter);
+    gtk_container_add(GTK_CONTAINER(
+            profilegui_widget(pg, "page_selector_container")), tvw);
+    gtk_widget_show(tvw);
+}
+
 /* Loads a Profile and creates a working dialog box for it */
 ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
 {
@@ -650,6 +845,8 @@ ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
     title = g_strdup_printf(_("ROXTerm Profile \"%s\""), profile_name);
     gtk_window_set_title(GTK_WINDOW(pg->widget), title);
     g_free(title);
+    
+    profilegui_setup_list_store(pg);
 
     g_hash_table_insert(profilegui_being_edited, g_strdup(profile_name), pg);
 
@@ -696,6 +893,7 @@ void profilegui_delete(ProfileGUI * pg)
         pg->ignore_destroy = TRUE;
         gtk_widget_destroy(profilegui_widget(pg, "Profile_Editor"));
     }
+    UNREF_LOG(g_object_unref(pg->list_store));
     UNREF_LOG(g_object_unref(pg->glade));
     dynamic_options_unref(dynopts, pg->profile_name);
     g_free(pg->profile_name);
