@@ -24,13 +24,17 @@
 #include "defns.h"
 #endif
 
+#include <vte/vte.h>
+
 #include "rtdbus.h"
 
 typedef struct ROXTermData ROXTermData;
 
-/* Make sure global options have been parsed before calling this. Sets some
- * stuff up before calling roxterm_launch. */
-void roxterm_init(const char *display_name);
+/* Make sure global options have been parsed before calling this, but call it
+ * before anything else like roxterm_launch. Sets up important global
+ * variables etc.
+ */
+void roxterm_init(void);
 
 /* Launch a new terminal in response to a D-BUS message (launcher) or for first
  * time (launcher = NULL); display_name may be NULL for default display. */
@@ -49,7 +53,27 @@ void roxterm_spawn(ROXTermData *, const char *command, ROXTerm_SpawnType);
 
 gboolean roxterm_spawn_command_line(const gchar *command_line,
         const char *display_name, const char *cwd, GError **error);
+
+VteTerminal *roxterm_get_vte_terminal(ROXTermData *roxterm);
+
+const char *roxterm_get_profile_name(ROXTermData *roxterm);
+
+const char *roxterm_get_colour_scheme_name(ROXTermData *roxterm);
         
+char *roxterm_get_cwd(ROXTermData *roxterm);
+
+/* Returns non-full-screen dimensions */
+void roxterm_get_nonfs_dimensions(ROXTermData *roxterm, int *cols, int *rows);
+
+double roxterm_get_zoom_factor(ROXTermData *roxterm);
+
+char const * const *roxterm_get_actual_commandv(ROXTermData *roxterm);
+
+#if ENABLE_SM
+gboolean roxterm_load_session(const char *xml, gssize len,
+        const char *client_id);
+#endif
+
 #endif /* ROXTERM_H */
 
 /* vi:set sw=4 ts=4 noet cindent cino= */
